from typing import List, Optional, Set, Iterator

from danger_python.danger import Violation


def violations(text_violations: str, touched_files: Set[str]) -> List[Violation]:
    if not text_violations:
        return []
    violations_list: Iterator[Violation] = filter(
        None, map(violation, text_violations.split("\n"))
    )
    return list(filter(lambda x: x.file_name in touched_files, violations_list))


def violation(text_violation: str) -> Optional[Violation]:
    if not text_violation:
        return None
    message = text_violation.split(":")[3][1:]
    file_name = text_violation.split(":")[0][2:]
    line = text_violation.split(":")[1]
    return Violation(message=message, file_name=file_name, line=line)



import subprocess

from danger_python.plugins import DangerPlugin

from danger_flake8.violation import violations


class DangerFlake8(DangerPlugin):
    def lint(self):
        result = subprocess.run(["flake8"], capture_output=True, text=True)
        touched_files = set(
            self.danger.git.modified_files + self.danger.git.created_files
        )
        violations_list = violations(result.stdout, touched_files)

        for violation in violations_list:
            self.warn(violation.message, violation.file_name, violation.line)
